@@ -1,16 +1,16 @@
 use std::io::Cursor;
 use std::mem;
 
-use Result;
-use tape::{Tape, Walue};
-use type2::{Number, Operation, Operator};
+use {Result, Tape};
+use type2::number;
+use type2::operation::{Operand, Operation, Operator};
 
 /// A program.
 pub struct Program<'l> {
     routine: Routine<'l>,
     global: &'l [Vec<u8>],
     local: &'l [Vec<u8>],
-    stack: Vec<Number>,
+    stack: Vec<Operand>,
     stems: usize,
 }
 
@@ -23,7 +23,7 @@ struct Routine<'l> {
 impl<'l> Program<'l> {
     /// Create a program.
     #[inline]
-    pub fn new(code: &'l [u8], global: &'l [Vec<u8>], local: &'l [Vec<u8>]) -> Program<'l> {
+    pub fn new(code: &'l [u8], global: &'l [Vec<u8>], local: &'l [Vec<u8>]) -> Self {
         Program {
             routine: Routine::new(code),
             global: global,
@@ -35,27 +35,35 @@ impl<'l> Program<'l> {
 
     /// Return the next operation.
     pub fn next(&mut self) -> Result<Option<Operation>> {
-        use type2::Operator::*;
+        use type2::operation::Operator::*;
 
         if try!(self.routine.done()) {
             return Ok(None);
         }
 
-        macro_rules! flush(() => (mem::replace(&mut self.stack, vec![])));
+        macro_rules! clear(
+            () => (mem::replace(&mut self.stack, vec![]));
+        );
         macro_rules! pop(
             () => (match self.stack.pop() {
                 Some(value) => value,
                 _ => raise!("expected an argument"),
             });
-            ($kind:ident) => (match self.stack.pop() {
-                Some(Number::$kind(value)) => value,
+            (bool) => (match self.stack.pop() {
+                Some(value) => value != 0.0,
+                _ => raise!("expected an argument"),
+            });
+            (i32) => (match self.stack.pop() {
+                Some(value) if value as i32 as Operand == value => value as i32,
                 _ => raise!("expected an argument of a different type"),
             });
         );
-        macro_rules! push(($argument:expr) => ({
-            let argument = $argument;
-            self.stack.push(argument);
-        }));
+        macro_rules! push(
+            ($argument:expr) => ({
+                let argument = $argument;
+                self.stack.push(argument);
+            });
+        );
         macro_rules! read(($index:expr) => ({
             let length = self.stack.len();
             if $index >= length {
@@ -68,7 +76,7 @@ impl<'l> Program<'l> {
         loop {
             code = try!(self.routine.peek::<u8>());
             match code {
-                0x1c | 0x20...0xff => push!(try!(self.routine.take())),
+                0x1c | 0x20...0xff => push!(try!(self.routine.take_operand())),
                 _ => break,
             }
         }
@@ -82,7 +90,7 @@ impl<'l> Program<'l> {
             RMoveTo | HMoveTo | VMoveTo | RLineTo | HLineTo | VLineTo |
             RRCurveTo | HHCurveTo | HVCurveTo | VHCurveTo | VVCurveTo |
             RCurveLine | RLineCurve | Flex | Flex1 | HFlex | HFlex1 => {
-                return Ok(Some((operator, flush!())));
+                return Ok(Some((operator, clear!())));
             },
 
             // Terminal operator
@@ -99,12 +107,12 @@ impl<'l> Program<'l> {
             // Hint operators
             HStem | VStem | HStemHM | VStemHM => {
                 self.stems += self.stack.len() >> 1;
-                return Ok(Some((operator, flush!())));
+                return Ok(Some((operator, clear!())));
             },
             HintMask | CntrMask => {
                 self.stems += self.stack.len() >> 1;
-                let _: Vec<u8> = try!(Walue::read(&mut *self.routine, (self.stems + 7) >> 3));
-                return Ok(Some((operator, flush!())));
+                let _: Vec<u8> = read_walue!(&mut *self.routine, (self.stems + 7) >> 3);
+                return Ok(Some((operator, clear!())));
             },
 
             // Arithmetic operators
@@ -131,11 +139,11 @@ impl<'l> Program<'l> {
                 push!(left);
             },
             Index => {
-                let i = pop!(Integer);
+                let i = pop!(i32);
                 push!(read!(if i >= 0 { i as usize } else { 0 }));
             },
             Roll => {
-                let (shift, span) = (pop!(Integer), pop!(Integer));
+                let (shift, span) = (pop!(i32), pop!(i32));
                 let length = self.stack.len();
                 if span < 0 {
                     raise!("found an invalid argument");
@@ -162,10 +170,19 @@ impl<'l> Program<'l> {
             Get => unimplemented!(),
 
             // Conditional operators
-            And => push!(pop!().and(pop!())),
-            Or => push!(pop!().or(pop!())),
-            Not => push!(!pop!()),
-            Eq => push!(pop!().equal(pop!())),
+            And => {
+                let (right, left) = (pop!(bool), pop!(bool));
+                push!(if left && right { 1.0 } else { 0.0 })
+            },
+            Or => {
+                let (right, left) = (pop!(bool), pop!(bool));
+                push!(if left || right { 1.0 } else { 0.0 })
+            },
+            Not => push!(if pop!(bool) { 0.0 } else { 1.0 }),
+            Eq => {
+                let (right, left) = (pop!(), pop!());
+                push!(if left == right { 1.0 } else { 0.0 })
+            },
             IfElse => {
                 let (right, left, no, yes) = (pop!(), pop!(), pop!(), pop!());
                 push!(if left <= right { yes } else { no });
@@ -173,7 +190,7 @@ impl<'l> Program<'l> {
 
             // Subroutine operators
             CallSubr | CallGSubr => {
-                let address = pop!(Integer);
+                let address = pop!(i32);
                 let mut routine = {
                     let subroutines = if operator == CallSubr {
                         &self.local
@@ -212,6 +229,11 @@ impl<'l> Routine<'l> {
     fn done(&mut self) -> Result<bool> {
         Ok(try!(Tape::position(&mut self.tape)) == self.size as u64)
     }
+
+    #[inline(always)]
+    fn take_operand(&mut self) -> Result<Operand> {
+        number::read(&mut self.tape)
+    }
 }
 
 deref! { Routine<'l>::tape => Cursor<&'l [u8]> }
