@@ -1,11 +1,11 @@
 use setup;
 
 macro_rules! operations(
-    ($($operator:ident => [$($number:expr),*],)*) => ({
-        use postscript::compact::{Number, Operator};
+    ($($operator:ident => [$($operand:expr),*],)*) => ({
+        use postscript::compact::operation::{Operand, Operator};
         use std::collections::HashMap;
         let mut operations = HashMap::new();
-        $(operations.insert(Operator::$operator, vec![$(Number::Integer($number)),*]);)*
+        $(operations.insert(Operator::$operator, vec![$($operand as Operand),*]);)*
         operations
     });
 );
@@ -44,12 +44,32 @@ fn encodings() {
     assert_eq!(vector.len(), 1);
     match &vector[0] {
         encoding @ &Encoding::Standard => {
-            assert_eq!(strings.get(encoding.get(42).unwrap()).unwrap(), "asterisk");
+            assert_eq!(ok!(strings.get(ok!(encoding.get(42)))), "asterisk");
         },
         _ => unreachable!(),
     }
 }
 
+#[test]
+fn global_dictionaries() {
+    let set = setup();
+    let vector = &set.global_dictionaries;
+
+    assert_eq!(vector.len(), 1);
+    assert_eq!(&*vector[0], &operations!(
+        Version => [709],
+        Notice => [710],
+        Copyright => [711],
+        FullName => [712],
+        FamilyName => [712],
+        Weight => [388],
+        FontBBox => [-178, -335, 1138, 918],
+        CharSet => [8340],
+        CharStrings => [8917],
+        Private => [65, 33671],
+    ));
+}
+
 #[test]
 fn global_subroutines() {
     let set = setup();
@@ -69,6 +89,18 @@ fn header() {
     assert_eq!(table.offset_size, 2);
 }
 
+#[test]
+fn local_dictionaries() {
+    use postscript::compact::operation::Operator;
+
+    let set = setup();
+    let vector = &set.local_dictionaries;
+
+    assert_eq!(vector.len(), 1);
+    assert_eq!(vector[0].len(), 13);
+    assert_eq!(ok!(vector[0].get(Operator::BlueScale)), &[0.0375]);
+}
+
 #[test]
 fn local_subroutines() {
     let set = setup();
@@ -87,44 +119,12 @@ fn names() {
     assert_eq!(&vector[0], "SourceSerifPro-Regular");
 }
 
-#[test]
-fn private_dictionaries() {
-    use postscript::compact::{Number, Operator};
-
-    let set = setup();
-    let vector = &set.private_dictionaries;
-
-    assert_eq!(vector.len(), 1);
-    assert_eq!(vector[0].len(), 13);
-    assert_eq!(vector[0].get(Operator::BlueScale).unwrap(), &[Number::Real(0.0375)]);
-}
-
 #[test]
 fn strings() {
     let set = setup();
     let index = &set.strings;
 
     assert_eq!(index.len(), 322);
-    assert_eq!(index.get(175).unwrap(), "Aring");
-    assert_eq!(index.get(500).unwrap(), "nine.tosf");
-}
-
-#[test]
-fn top_dictionaries() {
-    let set = setup();
-    let vector = &set.top_dictionaries;
-
-    assert_eq!(vector.len(), 1);
-    assert_eq!(&*vector[0], &operations!(
-        Version => [709],
-        Notice => [710],
-        Copyright => [711],
-        FullName => [712],
-        FamilyName => [712],
-        Weight => [388],
-        FontBBox => [-178, -335, 1138, 918],
-        CharSet => [8340],
-        CharStrings => [8917],
-        Private => [65, 33671],
-    ));
+    assert_eq!(ok!(index.get(175)), "Aring");
+    assert_eq!(ok!(index.get(500)), "nine.tosf");
 }
